package exnihilocreatio.registries;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import exnihilocreatio.json.CustomItemStackJson;
import exnihilocreatio.registries.manager.ExNihiloRegistryManager;
import exnihilocreatio.registries.manager.IHammerDefaultRegistryProvider;
import exnihilocreatio.util.ItemInfo;
import lombok.Getter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import org.apache.commons.io.IOUtils;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.*;

public class HammerRegistry {
    @Getter
    private static HashMap<ItemInfo, List<HammerReward>> registry = new HashMap<ItemInfo, List<HammerReward>>();
    private static Gson gson = new GsonBuilder().setPrettyPrinting().registerTypeAdapter(ItemStack.class, new CustomItemStackJson()).create();

    public static void loadJson(File file) {
        registry.clear();

        if (file.exists()) {
            try {
                FileReader fr = new FileReader(file);
                HashMap<String, ArrayList<HammerReward>> gsonInput = gson.fromJson(fr, new TypeToken<HashMap<String, ArrayList<HammerReward>>>() {
                }.getType());

                for (Map.Entry<String, ArrayList<HammerReward>> s : gsonInput.entrySet()) {
                    ItemInfo stack = new ItemInfo(s.getKey());
                    registry.put(stack, s.getValue());
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            registerDefaults();
            saveJson(file);
        }
    }

    public static void saveJson(File file) {
        FileWriter fw = null;
        try {
            fw = new FileWriter(file);

            gson.toJson(registry, fw);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            IOUtils.closeQuietly(fw);
        }
    }

    /**
     * Use register instead
     */
    @Deprecated
    public static void addHammerRecipe(IBlockState state, ItemStack reward, int miningLevel, float chance, float fortuneChance) {
        register(state, reward, miningLevel, chance, fortuneChance);
    }

    /**
     * Adds a new Hammer recipe for use with Ex Nihilo hammers.
     *
     * @param state         BlockStoneAxle State
     * @param reward        Reward
     * @param miningLevel   Mining level of hammer. 0 = Wood/Gold, 1 = Stone, 2 = Iron, 3 = Diamond. Can be higher, but will need corresponding tool material.
     * @param chance        Chance of drop
     * @param fortuneChance Chance of drop per level of fortune
     */
    public static void register(IBlockState state, ItemStack reward, int miningLevel, float chance, float fortuneChance) {
        register(state, reward, miningLevel, chance, fortuneChance, false);
    }

    public static void register(IBlockState state, ItemStack reward, int miningLevel, float chance, float fortuneChance, boolean wildcard) {
        ItemInfo key = new ItemInfo(state);
        if (wildcard)
            key.setMeta(-1);

        List<HammerReward> rewards = registry.get(key);

        if (rewards == null) {
            rewards = new ArrayList<>();
        }

        rewards.add(new HammerReward(reward, miningLevel, chance, fortuneChance));
        registry.put(key, rewards);
    }

    public static List<ItemStack> getRewardDrops(Random random, IBlockState block, int miningLevel, int fortuneLevel) {
        List<ItemStack> rewards = new ArrayList<ItemStack>();

        for (HammerReward reward : getRewards(block)) {
            if (miningLevel >= reward.getMiningLevel()) {
                if (random.nextFloat() <= reward.getChance() + (reward.getFortuneChance() * fortuneLevel)) {
                    rewards.add(reward.getStack().copy());
                }
            }
        }

        return rewards;
    }

    @SuppressWarnings("unchecked")
    public static List<HammerReward> getRewards(IBlockState block) {
        return registry.getOrDefault(new ItemInfo(block), Collections.EMPTY_LIST);
    }

    public static boolean registered(Block block) {
        return registry.containsKey(new ItemInfo(block.func_176223_P()));
    }

    public static void registerDefaults() {
        for (IHammerDefaultRegistryProvider provider : ExNihiloRegistryManager.getDefaultHammerRecipeHandlers()) {
            provider.registerHammerRecipeDefaults();
        }
    }

    // Legacy
    @Deprecated
    @SuppressWarnings("unchecked")
    public static ArrayList<HammerReward> getRewards(IBlockState state, int miningLevel) {
        List<HammerReward> mapList = registry.getOrDefault(new ItemInfo(state), Collections.EMPTY_LIST);
        ArrayList<HammerReward> ret = new ArrayList<HammerReward>();

        for (HammerReward reward : mapList) {
            if (reward.getMiningLevel() <= miningLevel)
                ret.add(reward);
        }

        return ret;
    }
}
